from pylab import *
from matplotlib import rc
from matplotlib.pyplot import figure, axes, semilogx, plot, loglog,\
    xlabel, ylabel, title, grid, savefig, show, clf
from math import pi, sqrt
from pyPdf import PdfFileReader, PdfFileWriter

rc('text', usetex=True)

def texify(seq):
    """
    Accepts a sequence of strings. Break up a string into math mode parts and
    roman parts, and wrap the math mode parts in dollar signs:

    'Note that $f$ is $C^\infty$ on $U$' becomes
    ['Note that ', '$f$', ' is ', '$C^\infty$', ' on ', '$U$']
    """
    def totex(word):
        if word and word[0] == '$':
            return word[1:-1] + '\ '
        else:
            return '\mathrm{%s}\ ' % word.replace(' ', '\ ')
    return '$%s$' % ''.join(map(totex, seq))

def make_plotter(plot_method):
    def plotter(data, labels, xaxis, yaxis, titles, outpath, bounds = None):
        """
        plotter(data, labels, xaxis, yaxis, titles, outpath)

        A 'TeX sequence' here is a sequence of strings that obeys the rules
        given in 'texify' above.

        Data is a list of tuples, where the first element is a list of x values
        and the second is a list of y values. Labels can either be empty, for no
        labels at all, or a list of length precisely equal to the length of data
        of TeX sequences.  xaxis, yaxis, and titles are all TeX
        sequences. outpath is a string -- silently overwrites!
        """
        clf()

        labels = [texify(lbl) for lbl in labels]

        if bounds:
            xlim(bounds[0][0], bounds[1][0])
            ylim(bounds[0][1], bounds[1][1])

        xlabel(texify(xaxis), fontsize=14)
        ylabel(texify(yaxis), fontsize=14)
        title(texify(titles), fontsize=18)

        if len(labels) > 0:
            for (x, y), lbl in zip(data, labels):
                plot_method(x, y, label = lbl)

            legend(tuple(labels), loc=2)

        else:
            for x, y in data:
                plot_method(x,y)

        grid(True)
        savefig(outpath, transparent=True)

    return plotter

regular_plot  = make_plotter(plot)
loglog_plot   = make_plotter(loglog)
semilogy_plot = make_plotter(semilogy)

def read_lattice(path):
    """
    Returns a dictionary mapping fields to lists of values.
    narea -> square root of area
    horiz -> horizontal dimension
    vert  -> vertical dimension
    npts  -> number of lattice points
    diff  -> normalized difference
    """
    file = open(path, 'r')
    narea = []
    horiz = []
    vert = []
    npts = []
    diff = []

    for line in file:
        nv, hv, vv, pv, dv = line.strip().split()
        narea.append(float(nv))
        horiz.append(float(hv))
        vert.append(float(vv))
        npts.append(float(pv))
        diff.append(float(dv))

    return {'narea' : narea,
            'horiz' : horiz,
            'vert'  : vert,
            'npts'  : npts,
            'diff'  : diff}

def collate(file_list, output):
    """
    Collates a list of paths to pdfs in order and writes the output
    to the output path.
    """
    writer = PdfFileWriter()

    for path in file_list:
        doc = PdfFileReader(file(path, 'rb'))

        for page in range(doc.getNumPages()):
            writer.addPage(doc.getPage(page))

    outstream = file(output, 'w')
    writer.write(outstream)
    outstream.close()

def rotate(pdf, output):
    """
    Rotates a pdf file by 90 degres and writes the output to the output path.
    """

    writer = PdfFileWriter()
    doc = PdfFileReader(file(pdf, 'rb'))

    for page in range(doc.getNumPages()):
        writer.addPage(doc.getPage(page).rotateClockwise(180))

    outstream = file(output, 'w')
    writer.write(outstream)
    outstream.close()

def bin(n, interval = 10):
    def lowest(point, interval):
        return interval * floor(float(point) / interval)

    data = [lowest(x, interval) for x in n]
    count = Counter(data)
    pairs = count.most_common()
    pairs.sort(key = lambda x : x[0])
    x, y = zip(*pairs)
    return (x, y)

def evenly(n, e_end):
    'Returns n evenly spaced numbers in [1, e_end]'
    return [1] if n == 1 else [1 + i * float(e_end - 1)/(n-1) for i in range(n)]

def linspace(n, e_start, e_end):
    'Returns n evenly spaced numbers in [e_start, e_end]'
    return [e_start] if n == 1 else [e_start + i * float(e_end-e_start)/(n-1) for i in range(n)]

def prefix_average(l):
    y0 = [0]
    for j in l:
        y0.append(y0[-1] + j)
    y0 = y0[1:]

    for i, _ in enumerate(y0):
        y0[i] = float(y0[i])/(i+1)

    return y0

def fst(x,y): return x
